var { createApp, ref, onMounted, h, defineComponent, computed, nextTick } = Vue
var ResultStorageKey = 'AuthResult'

var getQuery = url => {
  // str为？之后的参数部分字符串
  var str = url.substr(url.indexOf('?') + 1)
  // arr每个元素都是完整的参数键值
  var arr = str.split('&')
  // result为存储参数键值的集合
  var result = {}
  for (let i = 0; i < arr.length; i++) {
    // item的两个元素分别为参数名和参数值
    var item = arr[i].split('=')
    result[item[0]] = item[1]
  }
  return result
}
var params = getQuery(location.href)

var Axios = axios.create({
  baseURL: decodeURI(params.baseURL)
})

Axios.interceptors.request.use(requestConfig => {
  let token = localStorage.getItem('AccessToken')
  requestConfig.headers = {
    Authorization: `Bearer ${token}`
  }
  return requestConfig
})
var request = Axios.request

// webview.evalJs("webviewBridge('setToken')")
var StepOne = defineComponent({
  emits: ['success', 'error'],
  setup(_, { emit, expose }) {
    var rootRef = ref()
    var videoRef = ref()
    var videoMaskRef = ref()
    var videoTime = 2000

    var rootRefInfo = computed(() => ({
      width: rootRef.value.offsetWidth,
      height: rootRef.value.offsetHeight
    }))

    var userMedia = () => {
      return navigator.mediaDevices.getUserMedia({
        audio: false,
        video: true,
        width: rootRefInfo.value.width,
        height: rootRefInfo.value.height
      })
    }

    var videoPlay = mediaStream => {
      return new Promise(resolve => {
        videoRef.value.srcObject = mediaStream
        videoRef.value.play()
        videoRef.value.onloadedmetadata = function () {
          resolve()
        }
      })
    }

    var generateVideoFile = chunks => {
      var blob = new Blob(chunks, { type: 'video/mp4' })
      return new File([blob], 'face.mp4', { type: 'video/mp4' })
    }

    var transcribeVideo = (mediaStream, time) => {
      return new Promise(async resolve => {
        let timer = null
        var chunk = []
        var mediaRecorder = new MediaRecorder(mediaStream)

        mediaRecorder.ondataavailable = function (e) {
          chunk.push(e.data)
        }

        mediaRecorder.onstop = function () {
          var file = generateVideoFile(chunk)
          timer && clearTimeout(timer)
          resolve(file)
        }

        await videoPlay(mediaStream)

        mediaRecorder.start()
        videoMaskRef.value.style.background = 'transparent'

        timer = setTimeout(() => {
          mediaRecorder.stop()
        }, time)
      })
    }

    var startTranscribe = () => {
      userMedia()
        .then(async function (mediaStream) {
          if (mediaStream) {
            var file = await transcribeVideo(mediaStream, videoTime)
            mediaStream.getTracks().forEach(track => {
              track.stop() // 停止每个 track 的捕获
            })
            emit('success', file)
          }
        })
        .catch(() => {
          emit('error')
        })
    }

    expose({
      startTranscribe
    })

    return {
      rootRef,
      videoRef,
      videoMaskRef
    }
  },
  render() {
    return h(
      'div',
      {
        ref: 'rootRef',
        style: {
          width: '100vw',
          height: '100vh',
          padding: '100px 30px',
          boxSizing: 'border-box',
          overflow: 'hidden'
        }
      },
      [
        h(
          'div',
          {
            style: {
              position: 'relative',
              top: 0,
              left: 0,
              width: '100%',
              paddingBottom: '100%'
            }
          },
          [
            h('video', {
              ref: 'videoRef',
              style: {
                position: 'absolute',
                objectFit: 'cover',
                width: '100%',
                height: '100%',
                transform: 'rotateY(180deg)'
              },
              playsinline: true,
              contrals: true,
              'webkit-playsinline': true,
              muted: true
            }),
            h('div', {
              ref: 'videoMaskRef',
              style: {
                position: 'absolute',
                background: '#888',
                width: '100%',
                height: '100%',
                transition: '1s',
                borderRadius: '50%',
                boxShadow: 'inset 0px 0px 10px #fafafa'
              }
            }),
            h('img', {
              style: {
                position: 'absolute',
                maxWidth: '100%',
                width: '100%',
                height: '100%'
              },
              src: 'https://huifangzx-new.oss-cn-qingdao.aliyuncs.com/hfzx-mp/images/government/bgface.png'
            })
          ]
        ),
        h(
          'div',
          {
            style: {
              textAlign: 'center',
              color: '#191919',
              fontSize: '32rpx',
              marginTop: '45px'
            }
          },
          '眨眨眼'
        ),
        h('div', {
          style: {
            width: '100vw',
            height: '100vh',
            position: 'absolute',
            top: 0,
            left: 0,
            zIndex: 999
          }
        })
      ]
    )
  }
})
var StepTwo = defineComponent({
  render() {
    return h(
      'div',
      {
        style: {
          width: '100vw',
          height: '100vh',
          display: 'flex',
          alignItems: 'center',
          justifyContent: 'center'
        }
      },
      h('div', [
        [
          h(
            'div',
            {
              style: {
                width: '64px',
                height: '64px',
                margin: '0 auto'
              }
            },
            [
              h('img', {
                style: {
                  width: '100%',
                  height: '100%'
                },
                src: 'https://huifangzx-new.oss-cn-qingdao.aliyuncs.com/hfzx-mp/images/government/sbz_icon%402x.gif'
              })
            ]
          ),
          h(
            'div',
            {
              style: {
                textAlign: 'center'
              }
            },
            [
              h(
                'div',
                {
                  style: {
                    fontSize: '18px',
                    fontWeight: 500,
                    color: '#191919',
                    marginTop: '30px'
                  }
                },
                '验证中～'
              ),
              h(
                'div',
                {
                  style: {
                    fontSize: '14px',
                    color: '#888888',
                    marginTop: '10px'
                  }
                },
                '人脸识别中，请等待～'
              )
            ]
          )
        ]
      ])
    )
  }
})
var StepThree = defineComponent({
  props: {
    status: Number
  },
  emits: ['success', 'error'],
  setup(_, { emit }) {
    var triggerSuccess = () => emit('success')
    var triggerError = () => emit('error')

    return {
      triggerSuccess,
      triggerError
    }
  },
  render() {
    var { $props, triggerSuccess, triggerError } = this
    return h(
      'div',
      {
        style: {
          width: '100vw',
          height: '100vh',
          paddingTop: '100px',
          position: 'relative',
          boxSizing: 'border-box'
        }
      },
      [
        $props.status === 0
          ? h('div', [
              h(
                'div',
                {
                  style: {
                    width: '70vw',
                    margin: '0 auto'
                  }
                },
                [
                  h('img', {
                    style: {
                      width: '100%'
                    },
                    src: 'https://huifangzx-new.oss-cn-qingdao.aliyuncs.com/hfzx-mp/images/government/wrz_image%402x.png'
                  })
                ]
              ),
              h(
                'div',
                {
                  style: {
                    textAlign: 'center'
                  }
                },
                [
                  h(
                    'div',
                    {
                      style: {
                        fontSize: '18px',
                        fontWeight: 500,
                        color: '#191919',
                        marginTop: '30px'
                      }
                    },
                    '很遗憾实名认证失败～'
                  ),
                  h(
                    'div',
                    {
                      style: {
                        fontSize: '14px',
                        color: '#888888',
                        marginTop: '10px'
                      }
                    },
                    '请重新认证'
                  )
                ]
              ),
              h(
                'div',
                {
                  style: {
                    color: '#fff',
                    left: '16px',
                    right: '16px',
                    boxSizing: 'border-box',
                    background: '#FE7B3B',
                    borderRadius: '6px',
                    padding: '12px 0',
                    fontSize: '16px',
                    textAlign: 'center',
                    position: 'absolute',
                    bottom: '100px'
                  },
                  onClick: triggerError
                },
                '重新认证'
              )
            ])
          : null,
        $props.status === 1
          ? h('div', [
              h(
                'div',
                {
                  style: {
                    width: '70vw',
                    margin: '0 auto'
                  }
                },
                [
                  h('img', {
                    style: {
                      width: '100%'
                    },
                    src: 'https://huifangzx-new.oss-cn-qingdao.aliyuncs.com/hfzx-mp/images/government/yrz_image%402x.png'
                  })
                ]
              ),
              h(
                'div',
                {
                  style: {
                    textAlign: 'center'
                  }
                },
                [
                  h(
                    'div',
                    {
                      style: {
                        fontSize: '18px',
                        fontWeight: 500,
                        color: '#191919',
                        marginTop: '30px'
                      }
                    },
                    '恭喜您实名认证成功'
                  )
                ]
              ),
              h(
                'div',
                {
                  style: {
                    color: '#fff',
                    left: '16px',
                    right: '16px',
                    boxSizing: 'border-box',
                    background: '#2C53DA',
                    borderRadius: '6px',
                    padding: '12px 0',
                    fontSize: '16px',
                    textAlign: 'center',
                    position: 'absolute',
                    bottom: '100px'
                  },
                  onClick: triggerSuccess
                },
                '返回'
              )
            ])
          : null
      ]
    )
  }
})

var App = defineComponent({
  components: {
    StepOne
  },
  setup() {
    var currentStep = ref(0)
    var stepOneRef = ref()
    var authStatus = ref() // 1 认证中 2 成功 3 失败

    var closePage = () => {
      plus.webview.close('999')
    }
    var authSuccess = () => {
      plus.storage.setItem(ResultStorageKey, '1')
      closePage()
    }

    var authError = () => {
      plus.storage.setItem(ResultStorageKey, '0')
      closePage()
    }

    var reAuth = () => {
      currentStep.value = 0
      nextTick(() => {
        stepOneRef.value.startTranscribe()
      })
    }

    var transcribeSuccess = async file => {
      var formData = new FormData()
      formData.append('file', file)
      formData.append('motions', 'BLINK')
      formData.append('name', decodeURI(params.name))
      formData.append('certCode', decodeURI(params.code))
      try {
        var result = await request({
          url: '/user/cfca/auth',
          method: 'post',
          data: formData,
          onUploadProgress() {
            currentStep.value = 1
            authStatus.value = 1
          }
        })

        if (result.data.code === 0) {
          authStatus.value = 2
        } else {
          authStatus.value = 3
        }
      } catch (e) {
        console.log(e)
        authStatus.value = 3
      } finally {
        currentStep.value = 2
      }
    }

    onMounted(() => {
      currentStep.value === 0 && stepOneRef.value.startTranscribe()
    })

    return {
      currentStep,
      stepOneRef,
      authStatus,
      closePage,
      transcribeSuccess,
      authSuccess,
      authError,
      reAuth,
      params
    }
  },
  render() {
    var { currentStep, transcribeSuccess, authSuccess, reAuth, authStatus, authError } = this
    return h('div', [
      h(StepOne, {
        ref: 'stepOneRef',
        style: {
          display: currentStep === 0 ? 'block' : 'none'
        },
        onSuccess: transcribeSuccess,
        onError: authError
      }),
      currentStep === 1 ? h(StepTwo) : null,
      currentStep === 2
        ? h(StepThree, {
            status: authStatus === 2 ? 1 : 0,
            onSuccess: authSuccess,
            onError: reAuth
          })
        : null
    ])
  }
})

function webviewBridge(type, value) {
  if (type === 'setToken') {
    localStorage.setItem('AccessToken', value)
  }
}
window.webviewBridge = webviewBridge

setTimeout(() => {
  var app = createApp(App)
  app.mount('#app')
})
